/**
 * This file has no copyright assigned and is placed in the Public Domain.
 * This file is part of the mingw-w64 runtime package.
 * No warranty is given; refer to the file DISCLAIMER.PD within this package.
 */

#define FUNC __ms_wscanf_max_arg_count_internal
#define TYPE wchar_t
#include "scanf2-argcount-template.c"
